#!/usr/bin/env python

import argparse
from HTMLParser import HTMLParser
import os
import subprocess
import sys
import time

from codecs import open

from pelican.utils import slugify

def dc2fields(file):
    """Opens a Dotclear export file, and yield pelican fields"""
    try:
        from BeautifulSoup import BeautifulStoneSoup
    except ImportError:
        error = ('Missing dependency '
                 '"BeautifulSoup" required to import Dotclear files.')
        sys.exit(error)


    in_post = False
    in_comment = False
    posts = []
    comments = []
    posts_list = {}

    with open(file, 'r', encoding='utf-8') as f:

        for line in f:
            # remove final \n
            line = line[:-1]

            if line.startswith('[post '):
                in_post = True
            elif line.startswith('[comment'):
                in_comment = True
            elif in_post:
                if not line:
                    in_post = False
                else:
                    posts.append(line)
            elif in_comment:
                if not line:
                    in_comment = False
                else:
                  comments.append(line)

    print("%i posts read." % len(posts))
    print("%i comments read." % len(comments))

    for post in posts:
        fields = post.split('","')
        fields[0] = fields[0][1:]

        post_id = fields[0]
        post_slug = fields[11]

        posts_list[post_id]=post_slug
   
    for comment in comments:
        fields = comment.split('","')
        fields[0] = fields[0][1:]

        comment_id = fields[0]
        post_id = fields[1]
        comment_dt = fields[2]
        # comment_tz = fields[3]
        # comment_upddt = fields[4]
        comment_author = fields[5]
        comment_email = fields[6]
        comment_site = fields[7]
        comment_content = fields[8]
        # comment_words = fields[9]
        # comment_ip = fields[10]
        # comment_status = fields[11]
        # comment_spam_status = fields[12]
        # comment_spam_filter = fields[13]
        # comment_trackback = fields[14]

        
        if post_id:
            post_slug = posts_list[post_id].strip()

        yield (comment_id, post_slug, comment_dt, comment_author, comment_email, comment_site, comment_content)

def build_header(post_slug, comment_dt, comment_author, comment_email, comment_site):
    """Build a header from a list of fields"""
    header = ':post_id: %s\n' % post_slug
    if comment_author:
      if comment_site:
        header += ':Author: <a href="%s">%s</a>\n' % (comment_site, comment_author)
      else:
        header += ':Author: %s\n' % comment_author
      header += ':Name: %s\n' % comment_author
    if comment_dt:
        header += ':Date: %s\n' % comment_dt
    if comment_email:
        header += ':Email: %s\n' % comment_email
    if comment_site:
        header += ':Site: %s\n' % comment_site
    header += '\n'
    return header

def build_markdown_header(post_slug, comment_dt, comment_author, comment_email, comment_site):
    """Build a header from a list of fields"""
    header = 'post_id: %s\n' % post_slug
    if comment_author:
      if comment_site:
        header += 'Author: <a href="%s">%s</a>\n' % (comment_site, comment_author)
      else:
        header += 'Author: %s\n' % comment_author
      header += 'Name: %s\n' % comment_author
    if comment_dt:
        header += 'Date: %s\n' % comment_dt
    if comment_email:
        header += 'Email: %s\n' % comment_email
    if comment_site:
        header += 'Site: %s\n' % comment_site
    header += '\n'
    return header

def fields2pelican(fields, out_markup, output_path):
    for comment_id, post_slug, comment_dt, comment_author, comment_email, comment_site, comment_content in fields:
        filename = comment_id + "_" + post_slug
        if (out_markup == "rst") :
            ext = '.rst'
            header = build_header(post_slug, comment_dt, comment_author, comment_email, comment_site)
        else:
            out_markup = "markdown"
            ext = '.md'
            header = build_markdown_header(post_slug, comment_dt, comment_author, comment_email, comment_site)

        filename = os.path.basename(filename)

        # option to put files in directories with categories names
        out_filename = os.path.join(output_path, filename+ext)

        print(out_filename)

        
        html_filename = os.path.join(output_path, filename+'.html')

        with open(html_filename, 'w', encoding='utf-8') as fp:
            # Replace newlines with paragraphs wrapped with <p> so
            # HTML is valid before conversion
            paragraphs = comment_content.splitlines()
            #paragraphs = [u'<p>{0}</p>'.format(p) for p in paragraphs]
            new_content = ''.join(paragraphs)

            fp.write(new_content)


        parse_raw = '--parse-raw'
        cmd = ('pandoc --normalize --reference-links {0} --from=html'
               ' --to={1} -o "{2}" "{3}"').format(
                 parse_raw, out_markup, out_filename, html_filename)

        try:
            rc = subprocess.call(cmd, shell=True)
            if rc < 0:
                error = "Child was terminated by signal %d" % -rc
                exit(error)

            elif rc > 0:
                error = "Please, check your Pandoc installation."
                exit(error)
        except OSError, e:
            error = "Pandoc execution failed: %s" % e
            exit(error)

        os.remove(html_filename)

        with open(out_filename, 'r', encoding='utf-8') as fs:
            comment_content = fs.read()
            if out_markup == "markdown":
                # In markdown, to insert a <br />, end a line with two or more spaces & then a end-of-line
                comment_content = comment_content.replace("\\\\n ", "\n")
                comment_content = comment_content.replace("\\\\n", "\n")
                comment_content = comment_content.replace("\\n ", "\n")
                comment_content = comment_content.replace("\\n", "\n")
                comment_content = comment_content.replace("\\\n ", "\n")
                comment_content = comment_content.replace("\\\n", "\n")
                comment_content = comment_content.replace("\\\\r", "")

        with open(out_filename, 'w', encoding='utf-8') as fs:
            fs.write(header + comment_content)


def main():
    parser = argparse.ArgumentParser(
        description="Transform feed, Wordpress or Dotclear files to rst files."
            "Be sure to have pandoc installed",
        formatter_class=argparse.ArgumentDefaultsHelpFormatter)

    parser.add_argument(dest='input', help='The input file to read')
    parser.add_argument('-o', '--output', dest='output', default='output',
        help='Output path')
    parser.add_argument('-m', '--markup', dest='markup', default='markdown',
        help='Output markup format (supports rst & markdown)')

    args = parser.parse_args()

    if not os.path.exists(args.output):
        try:
            os.mkdir(args.output)
        except OSError:
            error = "Unable to create the output folder: " + args.output
            exit(error)

    fields = dc2fields(args.input)
    print(fields)

    fields2pelican(fields, args.markup, args.output)


main()